package com.spring.ai.imageclient.controller;

import com.spring.ai.imageclient.dto.PromptRequest;
import com.spring.ai.imageclient.dto.PromptResponse;
import org.apache.commons.lang3.StringUtils;
import org.springframework.ai.image.Image;
import org.springframework.ai.image.ImageClient;
import org.springframework.ai.image.ImagePrompt;
import org.springframework.ai.openai.OpenAiImageClient;
import org.springframework.ai.openai.OpenAiImageOptions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/api/v1/image")
public class ImageGenerationController {

    private final ImageClient imageClient;

    // constructor injection
    @Autowired
    public ImageGenerationController(OpenAiImageClient openAiImageClient) {
        this.imageClient = openAiImageClient;
    }

    /*
     * Generate an image based on the prompt
     * @param request - prompt request
     * @return prompt response
     */
    @PostMapping("/generate")
    public PromptResponse getImage(@RequestBody PromptRequest request) {
        final String prompt = request.getPrompt();

        // input validation
        if (StringUtils.isEmpty(prompt)) {
            System.out.println("Prompt is required");
            throw new IllegalArgumentException("Prompt is required");
        }

        System.out.println("Prompt: " + prompt);
        System.out.println("Generating image...");

        // image options
        OpenAiImageOptions imageOptions = OpenAiImageOptions.builder()
                .withQuality("standard")
                .withHeight(1024)
                .withN(1)
                .withWidth(1792)
                .build();

        // image prompt
        ImagePrompt imagePrompt = new ImagePrompt(prompt, imageOptions);

        System.out.println("Calling image client...");
        // class ep
        Image img = imageClient.call(imagePrompt).getResult().getOutput();

        System.out.println("Image generated successfully");
        return new PromptResponse(img.getUrl());
    }
}
